@@ -9,7 +9,7 @@ use IO::Socket;
 use IO::Select;
 use Net::IMAP::Simple::PipeSocket;
 
-our $VERSION = "1.2016";
+our $VERSION = "1.2018";
 
 BEGIN {
     # I'd really rather the pause/cpan indexers miss this "package"
@@ -59,11 +59,19 @@ sub new {
         $self->{cmd} = $1;
 
     } else {
-        my ( $srv, $prt ) = split( /:/, $server, 2 );
-        $prt ||= ( $opts{port} ? $opts{port} : $self->_port );
+        if( ($self->{server}, $self->{port}) = $server =~ m/^(\d{1,3}(?:\.\d{1,3}){3})(?::(\d+))?\z/ ) {
 
-        $self->{server} = $srv;
-        $self->{port}   = $prt;
+        } elsif( ($self->{server}, $self->{port}) = $server =~ m/^\[([a-fA-F0-9:]+)\]:(\d+)\z/ ) {
+
+        } elsif( ($self->{server}, $self->{port}) = $server =~ m/^([a-fA-F0-9:]+)\z/ ) {
+
+        } elsif( ($self->{server}, $self->{port}) = $server =~ m/^([^:]+):(\d+)\z/ ) {
+
+        } else {
+            $self->{server} = $server;
+        }
+
+        $self->{port} = $self->_port unless defined $self->{port};
     }
 
     $self->{timeout}           = ( $opts{timeout} ? $opts{timeout} : $self->_timeout );
@@ -75,13 +83,6 @@ sub new {
     $self->{debug}             = $opts{debug};
     $self->{readline_callback} = $opts{readline_callback};
 
-    # Pop the port off the address string if it's not an IPv6 IP address
-    if( $self->{server} ) {
-        if ( !$self->{use_v6} && $self->{server} =~ /^[A-Fa-f0-9]{4}:[A-Fa-f0-9]{4}:/ && $self->{server} =~ s/:(\d+)$//g ) {
-            $self->{port} = $1;
-        }
-    }
-
     my $sock;
     my $c;
     for ( my $i = 0 ; $i <= $self->{retry} ; $i++ ) {
@@ -1045,8 +1046,14 @@ sub _read_multiline {
     my $read_so_far = 0;
 
     while ( $read_so_far < $count ) {
-        push @lines, $sock->getline;
-        $read_so_far += length( $lines[-1] );
+        if( defined( my $line = $sock->getline ) ) {
+            $read_so_far += length( $line );
+            push @lines, $line;
+
+        } else {
+            $self->_seterrstr( "error reading $count bytes from socket" );
+            last;
+        }
     }
 
     if ( $self->{debug} ) {
@@ -1138,4 +1145,4 @@ sub _debug {
     return;
 }
 
-"True"; ## no critic -- pfft
+1;
